@@ -2,26 +2,24 @@ import argparse
 import os
 import platform
 import sys
-from typing import List, Union
+from typing import List, Optional, Tuple, Union
 
 import requests
 from pygments import __version__ as pygments_version
 from requests import __version__ as requests_version
 
 from httpie import __version__ as httpie_version
+from httpie.cli.constants import OUT_REQ_BODY, OUT_REQ_HEAD, OUT_RESP_BODY, OUT_RESP_HEAD
 from httpie.client import collect_messages
 from httpie.context import Environment
 from httpie.downloads import Downloader
-from httpie.output.writer import write_message, write_stream
+from httpie.output.writer import write_message, write_stream, MESSAGE_SEPARATOR_BYTES
 from httpie.plugins.registry import plugin_manager
 from httpie.status import ExitStatus, http_status_to_exit_status
 
 
 # noinspection PyDefaultArgument
-def main(
-    args: List[Union[str, bytes]] = sys.argv,
-    env=Environment(),
-) -> ExitStatus:
+def main(args: List[Union[str, bytes]] = sys.argv, env=Environment()) -> ExitStatus:
     """
     The main function.
 
@@ -111,64 +109,98 @@ def main(
     return exit_status
 
 
-def program(
+def get_output_options(
     args: argparse.Namespace,
-    env: Environment,
-) -> ExitStatus:
+    message: Union[requests.PreparedRequest, requests.Response]
+) -> Tuple[bool, bool]:
+    return {
+        requests.PreparedRequest: (
+            OUT_REQ_HEAD in args.output_options,
+            OUT_REQ_BODY in args.output_options,
+        ),
+        requests.Response: (
+            OUT_RESP_HEAD in args.output_options,
+            OUT_RESP_BODY in args.output_options,
+        ),
+    }[type(message)]
+
+
+def program(args: argparse.Namespace, env: Environment) -> ExitStatus:
     """
     The main program without error handling.
 
     """
+    # TODO: Refactor and drastically simplify, especially so that the separator logic is elsewhere.
     exit_status = ExitStatus.SUCCESS
     downloader = None
+    initial_request: Optional[requests.PreparedRequest] = None
+    final_response: Optional[requests.Response] = None
+
+    def separate():
+        getattr(env.stdout, 'buffer', env.stdout).write(MESSAGE_SEPARATOR_BYTES)
+
+    def request_body_read_callback(chunk: bytes):
+        should_pipe_to_stdout = bool(
+            # Request body output desired
+            OUT_REQ_BODY in args.output_options
+            # & not `.read()` already pre-request (e.g., for  compression)
+            and initial_request
+            # & non-EOF chunk
+            and chunk
+        )
+        if should_pipe_to_stdout:
+            msg = requests.PreparedRequest()
+            msg.is_body_upload_chunk = True
+            msg.body = chunk
+            msg.headers = initial_request.headers
+            write_message(requests_message=msg, env=env, args=args, with_body=True, with_headers=False)
 
     try:
         if args.download:
             args.follow = True  # --download implies --follow.
-            downloader = Downloader(
-                output_file=args.output_file,
-                progress_file=env.stderr,
-                resume=args.download_resume
-            )
+            downloader = Downloader(output_file=args.output_file, progress_file=env.stderr, resume=args.download_resume)
             downloader.pre_request(args.headers)
+        messages = collect_messages(args=args, config_dir=env.config.directory,
+                                    request_body_read_callback=request_body_read_callback)
+        force_separator = False
+        prev_with_body = False
 
-        initial_request = None
-        final_response = None
-
-        for message in collect_messages(args, env.config.directory):
-            write_message(
-                requests_message=message,
-                env=env,
-                args=args,
-            )
-            if isinstance(message, requests.PreparedRequest):
+        # Process messages as they’re generated
+        for message in messages:
+            is_request = isinstance(message, requests.PreparedRequest)
+            with_headers, with_body = get_output_options(args=args, message=message)
+            do_write_body = with_body
+            if prev_with_body and (with_headers or with_body) and (force_separator or not env.stdout_isatty):
+                # Separate after a previous message with body, if needed. See test_tokens.py.
+                separate()
+            force_separator = False
+            if is_request:
                 if not initial_request:
                     initial_request = message
+                    is_streamed_upload = not isinstance(message.body, (str, bytes))
+                    if with_body:
+                        do_write_body = not is_streamed_upload
+                        force_separator = is_streamed_upload and env.stdout_isatty
             else:
                 final_response = message
                 if args.check_status or downloader:
-                    exit_status = http_status_to_exit_status(
-                        http_status=message.status_code,
-                        follow=args.follow
-                    )
-                    if (not env.stdout_isatty
-                            and exit_status != ExitStatus.SUCCESS):
-                        env.log_error(
-                            f'HTTP {message.raw.status} {message.raw.reason}',
-                            level='warning'
-                        )
+                    exit_status = http_status_to_exit_status(http_status=message.status_code, follow=args.follow)
+                    if exit_status != ExitStatus.SUCCESS and (not env.stdout_isatty or args.quiet):
+                        env.log_error(f'HTTP {message.raw.status} {message.raw.reason}', level='warning')
+            write_message(requests_message=message, env=env, args=args, with_headers=with_headers,
+                          with_body=do_write_body)
+            prev_with_body = with_body
 
+        # Cleanup
+        if force_separator:
+            separate()
         if downloader and exit_status == ExitStatus.SUCCESS:
             # Last response body download.
             download_stream, download_to = downloader.start(
                 initial_url=initial_request.url,
                 final_response=final_response,
             )
-            write_stream(
-                stream=download_stream,
-                outfile=download_to,
-                flush=False,
-            )
+            write_stream(stream=download_stream, outfile=download_to, flush=False)
             downloader.finish()
             if downloader.interrupted:
                 exit_status = ExitStatus.ERROR
@@ -182,9 +214,7 @@ def program(
     finally:
         if downloader and not downloader.finished:
             downloader.failed()
-
-        if (not isinstance(args, list) and args.output_file
-                and args.output_file_specified):
+        if not isinstance(args, list) and args.output_file and args.output_file_specified:
             args.output_file.close()
 
 
@@ -212,6 +242,6 @@ def decode_raw_args(
     """
     return [
         arg.decode(stdin_encoding)
-        if type(arg) == bytes else arg
+        if type(arg) is bytes else arg
         for arg in args
     ]
