@@ -7,43 +7,52 @@ from httpie.plugins.base import AuthPlugin
 
 # noinspection PyAbstractClass
 class BuiltinAuthPlugin(AuthPlugin):
-
     package_name = '(builtin)'
 
 
 class HTTPBasicAuth(requests.auth.HTTPBasicAuth):
 
-    def __call__(self, r):
+    def __call__(
+        self,
+        request: requests.PreparedRequest
+    ) -> requests.PreparedRequest:
         """
         Override username/password serialization to allow unicode.
 
-        See https://github.com/jkbrzt/httpie/issues/212
+        See https://github.com/httpie/httpie/issues/212
 
         """
-        r.headers['Authorization'] = type(self).make_header(
+        # noinspection PyTypeChecker
+        request.headers['Authorization'] = type(self).make_header(
             self.username, self.password).encode('latin1')
-        return r
+        return request
 
     @staticmethod
-    def make_header(username, password):
+    def make_header(username: str, password: str) -> str:
         credentials = u'%s:%s' % (username, password)
         token = b64encode(credentials.encode('utf8')).strip().decode('latin1')
         return 'Basic %s' % token
 
 
 class BasicAuthPlugin(BuiltinAuthPlugin):
-
     name = 'Basic HTTP auth'
     auth_type = 'basic'
+    netrc_parse = True
 
-    def get_auth(self, username, password):
+    # noinspection PyMethodOverriding
+    def get_auth(self, username: str, password: str) -> HTTPBasicAuth:
         return HTTPBasicAuth(username, password)
 
 
 class DigestAuthPlugin(BuiltinAuthPlugin):
-
     name = 'Digest HTTP auth'
     auth_type = 'digest'
+    netrc_parse = True
 
-    def get_auth(self, username, password):
+    # noinspection PyMethodOverriding
+    def get_auth(
+        self,
+        username: str,
+        password: str
+    ) -> requests.auth.HTTPDigestAuth:
         return requests.auth.HTTPDigestAuth(username, password)
